@@ -9,11 +9,18 @@ import socket
 log = logging.getLogger('pystaggregator')
 
 class Client(Thread):
+    # This controls how many times the timeout value we will wait 
+    # in a worse case scenario when we are not able to connect to staggregator.
+    # if timeout is 1.00 second, then successive timeouts up to 1000
+    # seconds would be a period of about 5 days.
+    MAX_MULTIPLIER=1000
+
     def __init__(self, url, apikey, timeout=1.00):
         Thread.__init__(self)
         self.url = url                  # url of staggregator
         self.apikey = apikey            # api key for staggregator or None
         self.timeout = timeout          # timeout
+        self.original_timeout = timeout # holder for original value
         self.q = Queue()                # queue that holds outgoing messages
         self.should_stop = False        # do we keep running
         
@@ -26,6 +33,7 @@ class Client(Thread):
         self.daemon = True 
 
         self.headers = {'STAGGREGATOR_KEY':self.apikey}
+        self.session = requests.Session()
 
     def send(self, message):
         self.q.put(message)
@@ -43,16 +51,16 @@ class Client(Thread):
 
     def _build_message(self):
         m = []
-        timeout = 1.00 # seconds
         now = last_time = time.time()
         
         # this loop will get as many messages as the queue will
-        # provide in 50 ms.  Assuming that there are no more incoming,
-        # the timeout lets us say, "we're only going to wait a bit longer
-        # before sending what we have and then waiting on the queue again."
-        while last_time - now < timeout:
+        # provide in self.timeout seconds.  That means that
+        # this method will block for at most self.timeout seconds.
+        # at that point, it will return all of the messages
+        # it as accumulated
+        while last_time - now < self.timeout:
             try:
-                item = self.q.get(True, timeout)
+                item = self.q.get(True, self.timeout)
                 m.append(item)
             except Empty:
                 pass
@@ -63,24 +71,37 @@ class Client(Thread):
         log.info('sending message with {} stats'.format(len(message)))
         log.debug('message is {}'.format(message))
         try:
-            res = requests.post(self.url, headers=self.headers, 
+            res = self.session.post(self.url, headers=self.headers, 
                     data=json.dumps(message))
             res.raise_for_status()
+            # ok, we sent correctly, so restore our timeout value
+            self.timeout = self.original_timeout
         except Exception as e:
             log.error('could not send message {0}'.format(message))
             log.exception(e)
             # put these messages back in the queue, to be revisited when we
             # are connected again.
-            # we should probably back off the timeout for sending messages
-            # here
             for m in message:
                 log.debug('putting {0} back in queue'.format(m))
                 self.q.put(m)
+            # change our timeout for building messages so we 
+            # send more infrequently until connectivity is restored
+            if self.timeout <= self.original_timeout*self.MAX_MULTIPLIER:
+                self.timeout += self.timeout
+
 
 # a reference to the Client Thread
 _client = None
+# holding these lets us lazily start client threads, one per process
+# this helps things like bottlerocket work with gunicorn for example
+_start_args = None
 
-def start(url, key):
+def _start(url, key):
+    # save 'em for lazy start later
+    global _start_args
+    _start_args = (url, key)
+
+def _start(url, key):
     global _client
     log.info('starting client for {0} with apikey {1}'.format(url, key))
     _client = Client(url, key)
@@ -90,6 +111,9 @@ def send(message):
     """ send a correctly message to staggregator.
     :param message:  dictionary with name, value, and type fields
     """
+    if _client == None:
+        # lazy start client thread if needed
+        _start(*_start_args)
     _client.send(message)
 
 
